__version__ = "0.1.11"
# generated_by_export_class:logger_util||logger_ensure_logger
from .logger_util import logger_ensure_logger

# generated_by_export_class:logger_util||LoggerMultiLineLoggerAdapter
from .logger_util import LoggerMultiLineLoggerAdapter

# generated_by_export_class:ansible_util||ansible_get_line_kwargs
from .ansible_util import ansible_get_line_kwargs

# generated_by_export_class:common_util||common_first_or_none
from .common_util import common_first_or_none

# generated_by_export_class:common_util||common_first_or_raise
from .common_util import common_first_or_raise

# generated_by_export_class:common_util||common_generator_paginate
from .common_util import common_generator_paginate

# generated_by_export_class:mixin||MixinAssertMixin
from .mixin import MixinAssertMixin

# generated_by_export_class:mod_util||mod_record_map
from .mod_util import mod_record_map

# generated_by_export_class:mod_util||mod_get_in_db_set
from .mod_util import mod_get_in_db_set

# generated_by_export_class:mod_util||mod_record_cursor_id_between
from .mod_util import mod_record_cursor_id_between

# generated_by_export_class:request_util||request_tornado_repeat_request
from .request_util import request_tornado_repeat_request

# generated_by_export_class:request_util||request_requests_reapeat_request
from .request_util import request_requests_reapeat_request

# generated_by_export_class:jinja2_util||LocalFileSystemBytecodeCache
from .jinja2_util import LocalFileSystemBytecodeCache

# generated_by_export_class:city_util||province_and_city_dict
from .city_util import province_and_city_dict

# generated_by_export_class:escape_util||escape_recursive_unicode
from .escape_util import escape_recursive_unicode

# generated_by_export_class:escape_util||escape_to_basestring
from .escape_util import escape_to_basestring

# generated_by_export_class:escape_util||escape_to_unicode
from .escape_util import escape_to_unicode

# generated_by_export_class:escape_util||escape_utf8
from .escape_util import escape_utf8

# generated_by_export_class:escape_util||escape_parse_qs_bytes
from .escape_util import escape_parse_qs_bytes

# generated_by_export_class:escape_util||escape_url_unescape
from .escape_util import escape_url_unescape

# generated_by_export_class:escape_util||escape_url_escape
from .escape_util import escape_url_escape

# generated_by_export_class:escape_util||escape_squeeze
from .escape_util import escape_squeeze

# generated_by_export_class:escape_util||escape_json_decode
from .escape_util import escape_json_decode

# generated_by_export_class:escape_util||escape_json_encode
from .escape_util import escape_json_encode

# generated_by_export_class:escape_util||escape_xhtml_unescape
from .escape_util import escape_xhtml_unescape

# generated_by_export_class:escape_util||escape_xhtml_escape
from .escape_util import escape_xhtml_escape

# generated_by_export_class:escape_util||escape_native_str
from .escape_util import escape_native_str

# generated_by_export_class:city_and_loc_util||city_loc_list
from .city_and_loc_util import city_loc_list

__all__ = [
    'logger_ensure_logger',
    'LoggerMultiLineLoggerAdapter',
    'ansible_get_line_kwargs',
    'common_first_or_none',
    'common_first_or_raise',
    'common_generator_paginate',
    'MixinAssertMixin',
    'mod_record_map',
    'mod_get_in_db_set',
    'mod_record_cursor_id_between',
    'request_tornado_repeat_request',
    'request_requests_reapeat_request',
    'LocalFileSystemBytecodeCache',
    'province_and_city_dict',
    'escape_recursive_unicode',
    'escape_to_basestring',
    'escape_to_unicode',
    'escape_utf8',
    'escape_parse_qs_bytes',
    'escape_url_unescape',
    'escape_url_escape',
    'escape_squeeze',
    'escape_json_decode',
    'escape_json_encode',
    'escape_xhtml_unescape',
    'escape_xhtml_escape',
    'escape_native_str',
    'city_loc_list'
]





def mod_record_map(mod,
                   input_list,
                   field,
                   input_field=None):
    """
    一般用于表的关联查询,生成一个map
    比如user的detail信息 根据user_list, 提取用户id,也就是用户表的主键id, 然后在detail表里面找到user_id={user_id}的记录，然后做成一个map
        上面这个例子, mod:是Detail, input_list: userlist, input_field: id, field: user_id, 返回的是dict, key是field, value是mod的类型

    """
    if not input_list:
        return dict()
    if not isinstance(input_list[0], (str, int, float, bytes)):
        assert input_field is not None, "input_field should not be None when input_list[0] is instance of {}".format(type(input_list[0]))
        if isinstance(input_list[0], dict):
            input_list = [e[input_field] for e in input_list]
        else:
            input_list = [getattr(e, input_field) for e in input_list]
    query_set = mod.select().where([getattr(mod, field).in_ == input_list])
    return {getattr(e, field): e for e in query_set}


def mod_get_in_db_set(mod,
                      input_list,
                      field,
                      format=None):
    """
    找到input_list在数据库中存在的数据
    一般用于prepare_data里面
    用来替代:
        for name in input_name:
            if mod.get_or_none(name__eq=name):
                continue
        把多条sql转成一条

    """
    if not input_list:
        return set()

    def default_format(x):
        return getattr(x, field)
    if format is None:
        format = default_format
    query_set = mod.select().where([getattr(mod, field).in_ == input_list])
    return {format(x) for x in query_set}


def mod_record_cursor_id_between(mod, from_id, to_id,
                                 base_exression_list=None,
                                 page_size=10):
    """
    这个功能一般用户导出excel，可能要导入上w的数据，
    不使用这个的话，query_result.iterator() 第一个数据会卡住
    Args:
        from_id (int): >= from_id
        to_id (int): < to_id
        base_expression_list (list): expression_list
    """
    local_max_id = to_id
    if base_exression_list is None:
        base_exression_list = []
    while 1:
        new_query_result = mod.select().where(
            [getattr(mod, 'id') < local_max_id,
             getattr(mod, 'id') >= from_id,
             *base_exression_list]
        ).order_by(getattr(mod, 'id').desc()).paginate(0, page_size)

        if new_query_result.count() == 0:
            break

        for ele in new_query_result.iterator():
            yield ele
            local_max_id = min(local_max_id, ele.id)




async def request_tornado_repeat_request(request_fuc, retry_code_range_list=((599, 600)), max_request_count=5):
    from tornado.simple_httpclient import HTTPError
    # 为了安装这个包的时候，tornado不是必须的
    for i in range(max_request_count):
        res = await request_fuc()
        code = res.code
        is_match = False
        for code_range in retry_code_range_list:
            if code >= code_range[0] and code < code_range[1]:
                is_match = True
                break
        if is_match:
            if i < max_request_count - 1:
                continue
            else:
                raise HTTPError(res.code)
        else:
            return res


def request_requests_reapeat_request(request_func, max_request_count=5, timeout=(5, 20)):
    from requests.exceptions import ConnectionError, ReadTimeout
    for i in range(max_request_count):
        try:
            res = request_func()
            return res
        except ConnectionError:
            if i < max_request_count - 1:
                continue
            else:
                raise
        except ReadTimeout:
            if i < max_request_count - 1:
                continue
            else:
                raise
